<?php
include_once FCPATH . 'application/libraries/Cf_tcpdf.php';

class PdfComplianceReports_model extends Cf_Model
{
    private $outputPath;
    private $outputFilePath;
    private $fileName;
    private $complianceData;
    private $reportTitle;
    private $reportDescription;
    private $currentY;
    private $pdf;

    public function __construct()
    {
        parent::__construct();
        $this->outputPath = realpath(get_tmpdir());
    }

    public function generateReport(array $data)
    {
        [$this->fileName, $this->complianceData, $this->reportTitle, $this->reportDescription] = $data;

        !is_array($this->complianceData) &&
        $this->complianceData =  json_decode($this->complianceData, JSON_OBJECT_AS_ARRAY);

        $this->outputFilePath = "{$this->outputPath}/{$this->fileName}.pdf";

        $this->initPdf();
        $this->writeStatus('0');
        $this->pdf->printHeader($this->reportTitle, $this->reportDescription);
        $this->printCharts();
        $this->printBody();
        $this->savePDF();
        $this->writeStatus('100');
    }

    private function writeStatus($status)
    {
        file_put_contents("{$this->outputFilePath}.status", $status);
    }

    private function initPdf()
    {
        $this->pdf = new cf_tcpdf(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
        $this->pdf->SetTitle($this->reportTitle);
        $this->pdf->SetSubject($this->reportDescription);
        $this->pdf->SetPrintHeader(false);
        $this->pdf->AddPage();
    }

    private function printCharts()
    {
        $this->currentY = $this->pdf->getY();
        if (isset($this->complianceData['charts']) && !empty($this->complianceData['charts'])) {

            isset($this->complianceData['charts']['overall']) &&
            $this->pdf->ImageSVG('@' . $this->complianceData['charts']['overall'], $x = 0, $this->currentY, 120, 60);

            isset($this->complianceData['charts']['severity']) &&
            $this->pdf->ImageSVG('@' . $this->complianceData['charts']['severity'], $x = 100, $this->currentY, 120, 60);
            $this->currentY += 60;

            isset($this->complianceData['charts']['categories']) &&
            $this->pdf->ImageSVG('@' . $this->complianceData['charts']['categories'], $x = 15, $this->currentY, 120, 60);
            $this->currentY += 60;
        }

        $this->pdf->SetXY(15, $this->currentY);
    }

    private function printBody()
    {
        $html = $this->load->view('advancedreports/partials/compliance_report/export', $this->complianceData, true);
        $this->pdf->writeHTML($html, true, false, true, false, '');
    }

    private function savePDF()
    {
        $this->pdf->lastPage();
        $this->pdf->Output($this->outputFilePath, 'F');
    }
}
