<?php

namespace Controllers;

use Adldap\Objects\Paginator;
use models\compositers\UsersCompositor;
use models\dataMappers\UserMapper;
use models\LdapUser;
use models\LdapUserAdapter;
use models\UsersDataMapper;
use Services\Ldap\Implementations\Ldap;
use Slim\Http\Request;
use models\LdapConnection;

class UserController extends ProtectedController
{


    /**
     * @param Request $request
     * @param $response
     * @return mixed
     */
    public function getUserList(Request $request, $response)
    {
        $config = $this->container->get('ldap');

        $ldapService = new Ldap($config, new \Adldap\Connections\Provider(), new LdapConnection());

        $searchArr = [];
        if ($request->getParam('search')) {
            $searchArr = ['searchKey' => $config['login_attribute'], 'searchValue' => $request->getParam('search')];
        }

        $page = ($request->getParam('page') == 0 ? 0 : $request->getParam('page') - 1);
        $perPage = !$request->getParam('perPage') ? 25 : $request->getParam('perPage');

        $paginator = new Paginator
        (
            $ldapService->getUsersList($searchArr)->toArray(),
            $perPage,
            $page
        );

        $usersCompositor = new UsersCompositor($paginator->getIterator(), new LdapUserAdapter($config));
        $usersToReturn = $usersCompositor->map();


        $data = [
            'success' => true,
            'data' => $usersToReturn,
            'meta' => [
                'page' => $page + 1,
                'total' => $paginator->count(),
                'count' => $paginator->getPerPage(),
                'timestamp' => time(),
            ]
        ];

        $response = $response->withJson($data);
        return $response;
    }
}