'use strict';

reports.factory('complianceHelper', ['complianceWidgetHelper', function (complianceWidgetHelper) {
    let helper = {};

    const colors = {
        green: "#2ECC40",
        red: "#FF4136",
        orange: "#FF851B",
        yellow: "#FFDC00",
        gray: "#333333",
        lightGray: "#e2e2e2",
        white: "#fff",
        backgroundGray: "#21262a",
    };

    helper.overallConfig = function (stats) {
        if (stats.totalHosts == undefined) {
            return false;
        }

        let config = complianceWidgetHelper.getChartData(
            Math.floor(stats.total.pass / stats.totalChecks * 100), stats.totalChecks, stats.totalHosts
        );
        const label = {
            labels: {
                    items: [{
                        html: 'Overall compliance',
                        style: {"color": this.getChartsTextColor(), "fontSize": "15px", "fontWeight": "bold", left: '0px',
                            top: '0px'},
                        align: 'left'
                    }]
                }
        };
        config = {...config, ...label};
        config.plotOptions.pie.center = ['50%', '60%'];
        config.plotOptions.pie.size = '140px';

        config.title.align = 'center';
        config.title.x = 0;
        config.title.y = 35;

        return config;
    };

    helper.severityConfig = function (stats) {
        let config = {
            chart: {
                plotBackgroundColor: null,
                plotBorderWidth: 0,
                plotShadow: false,
                type: 'pie',
                backgroundColor: this.getBackgroundColor()
            },
            credits: {
                enabled: false
            },
            exporting: {enabled: false},
            title: {
                text: 'Failed checks by severity',
                style: {"color": this.getChartsTextColor(), "fontSize": "15px", "fontWeight": "bold", 'textTransform': 'none'},
                align: 'left'
            },
            plotOptions: {
                pie: {
                    borderWidth: 0,
                    dataLabels: {
                        formatter: function () {
                            if (this.y > 0) {
                                return `${this.point.name}: ${this.point.y}`;
                            }
                        },
                        color: this.getChartsTextColor(),
                        style: {
                            textOutline: 0
                        }
                    }
                }
            },
            tooltip: {
                pointFormat: 'Failed checks: <b>{point.y}</b>'
            },
            series: [{
                innerSize: '50%',
                size: '80%',
                data: [{
                    name: 'High',
                    y: stats.severity.high.fail,
                    color: colors.red,
                }, {
                    name: 'Medium',
                    y: stats.severity.medium.fail,
                    color: colors.orange,
                }, {
                    name: 'Low',
                    y: stats.severity.low.fail,
                    color: colors.yellow,
                }]
            }]
        };
        return config;
    };

    helper.categoriesConfig = function (stats) {
        let data = [];
        for (const category in stats.categories) {
            const val = Math.floor(stats.categories[category].totalScore / stats.categories[category].total);
            data.push({y: val, color: this.getColor(val), name: `${category}: ${val}%`});
        }

        let config = {
            chart: {
                type: 'bar',
                backgroundColor: this.getBackgroundColor()
            },
            exporting: {enabled: false},
            title: {
                text: 'Compliance by category',
                style: {"color": this.getChartsTextColor(), "fontSize": "15px", "fontWeight": "bold", 'textTransform': 'none'},
                align: 'left',

            },
            credits: {
                enabled: false
            },
            xAxis: {
                categories: data.map(item => item.name),
                labels: {
                    style: {
                        color: this.getChartsTextColor()
                    }
                }
            },
            yAxis: {
                min: 0,
                max: 100,
                title: {
                    text: ''
                }
            },
            legend: {
                enabled: false,
            },
            plotOptions: {
                series: {
                    borderWidth: 0,
                    stacking: 'normal'
                }
            },
            series: [{
                name: 'Compliance percentage',
                data: data,
            }]
        };
        return config;
    };

    helper.getColor = function (v) {
        let color = '';
        if (v >= 0 && v <= 20) {
            color = colors.red;
        } else if (v > 20 && v <= 50) {
            color = colors.orange;
        } else if (v > 50 && v <= 75) {
            color = colors.yellow;
        } else {
            color = colors.green
        }
        return color;
    };

    helper.getColorClass = function (v) {
        let className = '';
        if (v >= 0 && v <= 50) {
            className = 'important'
        } else if (v > 50 && v <= 75) {
            className = 'warning';
        } else {
            className = 'success';
        }
        return className;
    };

    helper.getChartsTextColor = function() {
        return  $('body').hasClass('dark') ? colors.lightGray : colors.gray;
    }

    helper.getBackgroundColor = function() {
        return  $('body').hasClass('dark') ? colors.backgroundGray : colors.white;
    }


    return helper;
}]);
