angular
    .module('complianceWidgetDirective', [])
    .directive('complianceWidget',
        [
            'complianceWidgetHelper', 'reportService', 'rulesService', '$q',
            function (complianceWidgetHelper, reportService, rulesService) {
    return {
        templateUrl: MP_SITE_URL + '/dashboard/partials/compliance_widget',
        restrict: 'A',
        scope: {
            widget: '=',
            currentUser: "=",
            currentDash: "=",
            saveOldName: '&',
            editName: '&',
            deleteWidget: '&',

        },
        link: function (scope, element) {
            scope.$watch(function () {
                return element.attr('refresh');
            }, function () {
                resizableWidget(element);
                scope.widget.chartList = complianceWidgetHelper.getChartData(scope.totalScore, scope.totalChecks, scope.totalCount);
            });

            reportService.loadReport(scope.widget.payload.complianceReport).then(
                function (result) {
                    let data = result.data;
                    let reportData = angular.fromJson(data.AdvancedReportsData.SQL ?? '');
                    scope.totalChecks = reportData.conditions.length;
                    scope.totalPass = 0;

                    reportService.getTotalHostsCount().then(count => {
                        scope.totalCount = count;
                        if (scope.totalChecks > 0) {
                            rulesService.getrules(reportData.conditions).then(res => {
                                scope.conditions = res.data;
                            });
                        }
                    })
                });

            scope.$watch('conditions', () => {
                if (scope.conditions !== undefined) {
                    let i = 0;
                    scope.conditions.forEach((condition, key) => {
                        rulesService.getStats(condition.id).then(function (data) {
                            i++;
                            if (data.data.failHosts == 0) {
                                delete scope.conditions[key];
                            } else {
                                scope.conditions[key].fail = data.data.failHosts;
                            }
                            scope.totalPass += data.data.failHosts > 0 ? 0 : 1;
                            if (i == scope.totalChecks) {
                                scope.totalScore = scope.totalPass == 0 ? 0 : Math.floor((scope.totalPass / scope.totalChecks) * 100);
                            }
                        });
                    })
                }
            });

            scope.$watch('totalScore', () => {
                if (scope.totalScore !== undefined) {
                    const severityMap = Object.keys(DCA.RuleSeverity).reduce((map, key) => {
                        map[DCA.RuleSeverity[key].value] = DCA.RuleSeverity[key].order;
                        return map;
                    }, {});
                    scope.conditions.sort((a, b) => {
                        if (severityMap[a.severity] > severityMap[b.severity]) {
                            return -1;
                        } else if (severityMap[a.severity] == severityMap[b.severity]) {
                            return b.fail - a.fail;
                        } else {
                            return 1;
                        }
                    }); // sort by severity and failed hosts
                    scope.conditions = scope.conditions.filter(a => a != undefined); // remove empty values
                    scope.widget.chartList = complianceWidgetHelper.getChartData(scope.totalScore, scope.totalChecks, scope.totalCount);
                }
            })
        }
    };
}]);
